use crate::diesel_setup::DbPool;
use diesel::RunQueryDsl;
use crate::models::{CharListDataset, CharListEntries, NewCharListEntries};
use crate::models::{EamAccount, NewEamAccount, UpdateEamAccount};
use crate::models::{EamGroup, NewEamGroup, UpdateEamGroup};
use crate::models::{NewAccount, NewClassStats, NewCharacter};
use crate::schema::char_list_entries::dsl::*;
use crate::schema::account::dsl::*;
use crate::schema::class_stats::dsl::*;
use crate::schema::character::dsl::*;
use crate::schema::EamAccount as eam_accounts;
use crate::schema::EamAccount::dsl::*;
use crate::schema::EamGroup as eam_groups;
use crate::schema::EamGroup::dsl::*;
use diesel::insert_into;
use diesel::prelude::*;
use uuid::Uuid;

//########################
//#    CharListDataset   #
//########################
pub fn insert_char_list_dataset( pool: &DbPool, dataset: CharListDataset ) -> Result<usize, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");
    let mut entry = CharListEntries::from(dataset.clone());
    let entry_uuid = Uuid::new_v4().to_string();
    let entry_uuid_opt = Some(entry_uuid.clone());
    entry.id = Some(entry_uuid.clone());
    let new_entry = NewCharListEntries::from(entry);    
    
    //char_list_entries
    let result = insert_into(char_list_entries)
        .values(&new_entry)
        .execute(&mut conn);
    if result.is_err() {
        return Err(result.unwrap_err());
    }

    //account
    let mut acc = NewAccount::from(dataset.account);
    acc.entry_id = Some(entry_uuid.clone());
    let result = insert_into(account)
        .values(acc)
        .execute(&mut conn);
    if result.is_err() {
        return Err(result.unwrap_err());
    }

    //class_stats
    dataset.class_stats.iter().for_each(|class_stat| {
        let entry_uuid_opt_clone = entry_uuid_opt.as_ref().map(|s| s.clone());
        insert_into(class_stats)
            .values(NewClassStats::from_class_stats(class_stat, entry_uuid_opt_clone))
            .execute(&mut conn)
            .expect("Failed to insert class_stats");
    });

    //character
    dataset.character.iter().for_each(|chara| {
        let entry_uuid_opt_clone = entry_uuid_opt.clone();
        insert_into(character)
            .values(NewCharacter::from_character(chara, Some(entry_uuid_opt_clone.expect("Panick: entry_uuid_opt_clone is none"))))
            .execute(&mut conn)
            .expect("Failed to insert character");
    });

    Ok(0)
}

//########################
//#      EamAccount      #
//########################

pub fn get_all_eam_accounts(pool: &DbPool) -> Result<Vec<EamAccount>, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");
    EamAccount.load::<EamAccount>(&mut conn)
}

pub fn get_eam_account_by_email(
    pool: &DbPool,
    account_email: String,
) -> Result<EamAccount, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");
    eam_accounts::table.find(account_email).first(&mut conn)
}

pub fn insert_or_update_eam_account(
    pool: &DbPool,
    eam_account: EamAccount,
) -> Result<usize, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");

    // Get the max id and increment it by 1
    let max_id: Option<Option<i32>> = eam_accounts::table
        .select(diesel::dsl::max(eam_accounts::id))
        .first(&mut conn)
        .optional()?;

    let new_id = max_id.unwrap_or(None).unwrap_or(0) + 1; // If no max id, start at 1

    let mut clone_acc = eam_account.clone();
    clone_acc.id = Some(new_id);
    let insertable = NewEamAccount::from(clone_acc.clone());
    let updatable = UpdateEamAccount::from(eam_account);

    insert_into(eam_accounts::table)
        .values(&insertable)
        .on_conflict(eam_accounts::email)
        .do_update()
        .set(&updatable)
        .execute(&mut conn)
}

pub fn delete_eam_account(
    pool: &DbPool,
    account_email: String,
) -> Result<usize, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");
    diesel::delete(eam_accounts::table.find(account_email)).execute(&mut conn)
}

//########################
//#       EamGroup       #
//########################

pub fn get_all_eam_groups(pool: &DbPool) -> Result<Vec<EamGroup>, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");
    EamGroup.load::<EamGroup>(&mut conn)
}

pub fn insert_or_update_eam_group(
    pool: &DbPool,
    eam_group: EamGroup,
) -> Result<usize, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");

    let insertable: NewEamGroup = NewEamGroup::from(eam_group.clone());
    let updatable: UpdateEamGroup = UpdateEamGroup::from(eam_group.clone());

    diesel::insert_into(eam_groups::table)
        .values(&insertable)
        .on_conflict(eam_groups::id)
        .do_update()
        .set(&updatable)
        .execute(&mut conn)
}

pub fn delete_eam_group(pool: &DbPool, group_id: i32) -> Result<usize, diesel::result::Error> {
    let mut conn = pool.get().expect("Failed to get connection from pool.");
    diesel::delete(eam_groups::table.find(group_id)).execute(&mut conn)
}
